#include "moows/colorcombobox.h"
#include <QColorDialog>
#include <QPainter>
#include <QFontMetrics>

using namespace moo::ws;

ColorComboBox::ColorComboBox(QWidget *parent) :
    QComboBox(parent),
    n_custom_colors(0),
    custom_enabled(false),
    show_names(true)
{
    connect(this, SIGNAL(activated(int)), SLOT(activated(int)));
}

void ColorComboBox::setColorDialogEnabled(bool enabled)
{
    if (custom_enabled == enabled)
	return;

    custom_enabled = enabled;

    if (enabled)
	addItem(tr("Other..."));
    else
        removeItem(colorCount());
}

bool ColorComboBox::colorDialogEnabled() const
{
    return custom_enabled;
}


const int pix_size = 12;

void ColorComboBox::insertColor(int index, const QColor &color, const QString &name)
{
    QPixmap pix(pix_size, pix_size);
    QPainter painter(&pix);

    if (color.isValid())
    {
	painter.setPen(Qt::gray);
	painter.setBrush(QBrush(color));
	painter.drawRect(0, 0, pix_size, pix_size);
    }

    QIcon icon;
    icon.addPixmap(pix);

    if (custom_enabled && index > colorCount())
        index = colorCount() - 1;

    insertItem(index, icon, name, color);
}

int ColorComboBox::colorCount() const
{
    return QComboBox::count() - (custom_enabled ? 1 : 0);
}

QColor ColorComboBox::currentColor() const
{
    return color(currentIndex());
}

void ColorComboBox::setCurrentColor(const QColor &newColor)
{
    for (int i = 0; i < colorCount(); i++)
    {
	if (color(i) == newColor)
        {
	    setCurrentIndex(i);
	    break;
	}
    }
}

QColor ColorComboBox::color(int index) const
{
    if (index == colorCount())
	return QColor();
    else
        return qVariantValue<QColor>(itemData(index));
}

void ColorComboBox::activated(int index)
{
    QColor newColor;

    if (custom_enabled && index == colorCount())
    {
	newColor = QColorDialog::getColor();

        if (newColor.isValid())
        {
            n_custom_colors += 1;
	    addColor(newColor, tr("Custom %1").arg(n_custom_colors));
	    setCurrentIndex(index);
	}
        else
        {
	    setCurrentColor(last);
	    newColor = last;
	}
    }
    else
    {
        newColor = color(index);
    }

    if (newColor != last)
    {
        last = newColor;
        emit activated(newColor);
    }
}

QSize ColorComboBox::sizeHint() const
{
    QFontMetrics metrics = fontMetrics();
    QStyleOptionComboBox option;
    option.init(this);
    return style()->sizeFromContents(QStyle::CT_ComboBox, &option,
                                     QSize(metrics.width(tr("Custom 99")) + pix_size + 8, metrics.height() + 4),
                                     this);
}

void ColorComboBox::setStandardColors()
{
    addColor(Qt::black, tr("Black"));
    addColor(Qt::white, tr("White"));
    addColor(Qt::red, tr("Red"));
    addColor(Qt::darkRed, tr("Dark red"));
    addColor(Qt::green, tr("Green"));
    addColor(Qt::darkGreen, tr("Dark green"));
    addColor(Qt::blue, tr("Blue"));
    addColor(Qt::darkBlue, tr("Dark blue"));
    addColor(Qt::cyan, tr("Cyan"));
    addColor(Qt::darkCyan, tr("Dark cyan"));
    addColor(Qt::magenta, tr("Magenta"));
    addColor(Qt::darkMagenta, tr("Dark magenta"));
    addColor(Qt::yellow, tr("Yellow"));
    addColor(Qt::darkYellow, tr("Dark yellow"));
    addColor(Qt::gray, tr("Gray"));
    addColor(Qt::darkGray, tr("Dark gray"));
    addColor(Qt::lightGray, tr("Light gray"));
}
